'''
${name} xform.
'''

import logging

from fashion.xformUtil import readModels
from fashion.xformUtil import generate

def outputKinds():
    '''Outputs generated by this xform'''
    return [ 'fashion_gen' ]

def police():
    '''Validate input combinations.'''
    pass

def xform():
    '''Generate many source files from 2 input models.'''
    logging.debug("${name} xform running")
    
    model = {  }
    
    generate(model, "${template}", "${target}")
